@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Utility functions for doing data analysis, such as wrappers for functions
 # provided by pandas and sklearn.
@@ -11,6 +11,9 @@
 #
 # TODO:
 # - Add simple wrapper class for commonly used idioms, such as len(df.columns) for number of columns.
+# TODO2:
+# - * Add wrappers for common tasks that are awkward and/or unintuitive in pandas (e.g., arbitrary cell access)!
+# - Similarly, add a simple cheatsheet.
 #
 
 """Utility functions for work with data (e.g., pandas wrappers)"""
@@ -27,8 +30,10 @@ from mezcla.text_utils import version_to_number as version_as_float
 # Constants
 # Note: Delim defaults to None so that dialect inference can be used.
 # This is a quirk with Pandas compared to the cvs module.
-DELIM = system.getenv_value("DELIM", None,
-                            "Delimiter for input and output tables")
+## TODO3: rename to CVS_DELIM to avoid conflict with other usage
+DELIM = system.getenv_value(
+    "DELIM", None,
+    desc="Delimiter for input and output tables")
 COMMENT = 'comment'
 DIALECT = 'dialect'
 EXCEL = 'excel'
@@ -46,30 +51,45 @@ if (version_as_float(pd.__version__) < version_as_float(MIN_PANDAS_VERSION)):
 
 def read_csv(filename, **in_kw):
     """Wrapper around pandas read_csv
-    Note: delimiter SEP defaults to DELIM env. var, dtype to str, and both error_bad_lines & keep_default_na to False. (Override these via keyword paramsters.)
+    Note: delimiter SEP defaults to DELIM env. var (n.b., uses sniffing if unset), dtype to str, and both error_bad_lines & keep_default_na to False. (Override these via keyword parameters.)
     """
-    # EX: tf = read_csv("examples/iris.csv"); tf.shape => (150, 5)
+    ## TODO2: make the defaults more unintuitive and add option to disable
+    # EX: df = read_csv("examples/iris.csv"); df.shape => (150, 5)
     ## TODO: clarify dtype usage
     kw = {SEP: DELIM, 'dtype': str,
-          ## BAD: 'error_bad_lines': False, 'keep_default_na': False}
           'on_bad_lines': 'skip', 'keep_default_na': False}
+    # Hack: make sure only one of "sep" and "delimiter" specified.
+    # Note: delimiter is now an alias for sep.
+    if DELIMITER in in_kw:
+        debug.assertion(not (kw[SEP] and in_kw[DELIMITER]))
+        kw[SEP] = (kw[SEP] or in_kw[DELIMITER])
+        in_kw[DELIMITER] = None
+    # Use comma (tab) for delimiter if unspecified and file ext is CSV (TSV)
+    if not kw[SEP]:
+        if filename.lower().endswith(".csv"):
+            kw[SEP] = ","
+        if filename.lower().endswith(".tsv"):
+            kw[SEP] = "\t"
     # Overide settings based on explicit keyword arguments
     kw.update(**in_kw)
-    # Turn off quotoing if tab delimited
+    kw['engine'] = 'python'
+    # Turn off quoting if tab delimited
     if kw[SEP] == "\t":
         kw['quoting'] = csv.QUOTE_NONE
     # Add special processing (n.b., a bit idiosyncratic)
     if ((COMMENT not in kw) and (kw.get(DIALECT) != EXCEL)):
-        debug.trace_fmt(4, "Enabling comments in read_csv")
+        debug.trace_fmt(4, "FYI: Enabling comments in data_utils.read_csv")
         kw[COMMENT] = "#"
-    debug.trace_fmt(5, "read_csv({f}, [in_kw={ikw}])", f=filename, ikw=in_kw)
-    debug.trace_fmt(6, "\tkw={k}", k=kw)
+    debug.trace_fmt(4, "in data_utils.read_csv({f}, [in_kw={ikw}])", f=filename, ikw=in_kw)
+    debug.trace_fmt(4, "\tFYI: kw={k}", k=kw)
     df = None
     try:
         df = pd.read_csv(filename, **kw)
+        debug.assertion(not(any(c.startswith(" ") for c in list(df.columns))),
+                        'make sure CSV FILE delim not ", "')
     except:
-        debug.trace(4, f"Exception during read_csv: {system.get_exception()}")
-    debug.trace(4, f"read_csv({filename}) => {df}")
+        debug.trace(3, f"Exception during read_csv: {system.get_exception()}")
+    debug.trace(4, f"read_csv({filename}) =>\n{df}")
     return df
 
 
@@ -80,11 +100,13 @@ def to_csv(filename, data_frame, **in_kw):
     result = None
     kw = {SEP: DELIM, 'index': False}
     kw.update(**in_kw)
+    debug.trace_fmt(4, "in data_utils.to_csv({f}, _df) [in_kw={ikw}])", f=filename, ikw=in_kw)
+    debug.trace_fmt(4, "\tFYI: kw={k}", k=kw)
     try:
         result = data_frame.to_csv(filename, **kw)
     except:
         debug.trace(4, f"Exception during write_csv: {system.get_exception()}")
-    debug.trace(4, f"to_csv({filename}, {data_frame}) => {result}")
+    debug.trace(5, f"data_utils.to_csv({filename}, {data_frame}) => {result}")
     return result
 #
 write_csv = to_csv
@@ -92,7 +114,7 @@ write_csv = to_csv
 
 def lookup_df_value(data_frame, return_field, lookup_field, lookup_value):
     """Return value for DATA_FRAME's RETURN_FIELD given LOOKUP_FIELD value LOOKUP_VALUE"""
-    # EX: lookup_df_value(tf, "sepal_length", "petal_length", "3.8") => "5.5"
+    # EX: lookup_df_value(df, "sepal_length", "petal_length", "3.8") => "5.5"
     ## TODO: rework in terms of Pandas primitives
     value = None
     try:
@@ -109,7 +131,7 @@ def lookup_df_value(data_frame, return_field, lookup_field, lookup_value):
 
 def main():
     """Entry point for script"""
-    system.print_stderr("Error: Not intended to being invoked directly")
+    system.print_stderr("Error: Not intended to be invoked directly")
     return
 
 #-------------------------------------------------------------------------------
