@@ -1,10 +1,10 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # format-profile.py: displays the result of python profiling via cProfile
 #
 # usage example:
 #    python -u -m cProfile -o fubar.profile fubar.py
-#    format-profile.py fubar.profile >| fubar.profile.list
+#    format-profile.py fubar.profile > fubar.profile.list
 #------------------------------------------------------------------------
 # Notes:
 # via http://docs.python.org/3/library/profile.html:
@@ -26,9 +26,7 @@
 #
 # Valid Arg    Meaning
 # 'calls'      call count
-# 'cumulative' cumulative time
 # 'cumtime'    cumulative time
-# 'file'       file name
 # 'filename'   file name
 # 'module'     file name
 # 'ncalls'     call count
@@ -37,12 +35,8 @@
 # 'name'       function name
 # 'nfl'        name/file/line
 # 'stdname'    standard name
-# 'time'       internal time
 # 'tottime'    internal time
 #
-# Note: following apply to version 2.7 or later
-# - cumtime, filename, ncalls, tottime
-#
 
 """Displays result of python profiling"""
 
@@ -55,12 +49,15 @@ import pstats
 
 # Local packages
 from mezcla import debug
-## OLD: from tpo_common import *
-from mezcla.system import getenv_text, print_stderr
+from mezcla.system import getenv_bool, getenv_text, print_stderr
 
-## OLD: PROFILE_KEY = getenv_text("PROFILE_KEY", "cumulative")
-PROFILE_KEY = getenv_text("PROFILE_KEY", "cumulative",
-                          "Sort key (e.g., calls, cumulative, file, time)")
+# Env. Constants
+PROFILE_KEY = getenv_text(
+    "PROFILE_KEY", "cumulative",
+    desc="Sort key (e.g., cumtime, filename, ncalls, tottime)")
+FULL_PATH = getenv_bool(
+    "FULL_PATH", False,
+    desc="Show full path in filename field")
 
 #------------------------------------------------------------------------
 # Functions
@@ -68,24 +65,25 @@ PROFILE_KEY = getenv_text("PROFILE_KEY", "cumulative",
 def usage():
     """Displays usage notes for script"""
     print_stderr("""
-
 Usage: {program} profile-log
 
 Notes:
-- use PROFILE_KEY to over default sorting (cumulative)
-- main keys: 
-       calls, cumulative, file, time
-- other keys: 
-       module, pcalls, line, name, nfl, stdname
-- 2.7+ keys: 
+- Use FULL_PATH to include directory for filename
+  (e.g., helps to resolve all those __init__.py entries).
+- Use PROFILE_KEY to over default sorting (cumulative).
+- Main keys: 
        cumtime, filename, ncalls, tottime
-- unfortunately, memory profiling is not supported
-- see http://docs.python.org/3/library/profile.html
+- Other keys: 
+       module, pcalls, line, name, nfl, stdname
+- Alternative keys:
+       calls, cumulative, file, time
+- Unfortunately, memory profiling is not supported.
+- For more details, check following:
+    http://docs.python.org/3/library/profile.html
 
 Example (assumes bash):
-    $ python -m cProfile -o profile.log fubar.py
-    $ PROFILE_KEY=calls {program} profile.log
-
+    $ python -m cProfile -o /tmp/profile.data simple_main_example.py
+    $ PROFILE_KEY=calls {program} /tmp/profile.data | head
 """.format(program=sys.argv[0]))
     return
                 
@@ -100,7 +98,9 @@ def main():
 
     # Generate listing and sort by cumulative time
     p = pstats.Stats(file)
-    p.strip_dirs().sort_stats(PROFILE_KEY).print_stats()
+    if not FULL_PATH:
+        p = p.strip_dirs()
+    p.sort_stats(PROFILE_KEY).print_stats()
     return
 
 #------------------------------------------------------------------------
