@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # gensim_test.py: Tests features of gensim document processing, such ad TF/IDF-based term vector
 # representation and documented similarity computations. This takes as input a text file where each
@@ -17,6 +17,10 @@
 # - Use token quoting consistent with Gensim topic display (i.e., double).
 # - Work around quirk requiring the model to be saved prior to similarity calculations.
 # 
+# TODO2:
+# - Check gensim 4.0 API usage: https://github.com/piskvorky/gensim/wiki/Migrating-from-Gensim-3.x-to-4.
+# - Fix similarity when user doc ID's provided.
+# 
 
 """Interface into Gensim package for vector-based text analysis (e.g., document similarity)"""
 
@@ -33,9 +37,9 @@ import tempfile
 from gensim import corpora, models, similarities
 
 # Local packages
-import mezcla.system as system
+from mezcla import system
 import mezcla.tpo_common as tpo
-import mezcla.debug as debug
+from mezcla import debug
 import mezcla.glue_helpers as gh
 
 # Environment options
@@ -169,6 +173,7 @@ class CorpusData(object):
         ## OLD: """Returns iterator over vectors in corpus or over lines in input text"""
         tpo.debug_print("CorpusData.__iter__()", 6)
         if (self.mm):
+            # pylint: disable=use-yield-from
             for vector in self.mm.__iter__():
                 yield vector
         else:
@@ -405,7 +410,7 @@ def create_ordered_lookup_table(filename):
     lookup_hash = OrderedDict()
     f = None
     try:
-        f = open(filename)
+        f = system.open_file(filename)
         line_num = 0
         for line in f:
             lookup_hash[str(line_num)] = line.strip()
@@ -507,7 +512,7 @@ def main(runtime_args=None):
         ## temp_file = tpo.getenv_text("TEMP_FILE", tempfile.NamedTemporaryFile().name)
         temp_file = TEMP_BASE
         filename = temp_file + ".txt"
-        gh.write_file(filename, gh.read_file(None))
+        system.write_file(filename, gh.read_file(None))
 
     # Derive the basename if not given
     # TODO: TFIDF_MODEL_EXT = ".tfidf.mm"
@@ -649,3 +654,6 @@ if __name__ == '__main__':
     ## OLD:
     main()
     ## TODO: main(sys.argv)
+
+## TEST (for testing tests/test_gensim_test.py conditional loading):
+## fubar
