@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Use scikit-learn to perform probabilistic grid search for number of hidden
 # layers and number of units per layer.
@@ -25,12 +25,13 @@
 ## BAD: tensorflow.set_random_seed(7919)
 ## tensorflow.random.set_seed(7919)
 
-# Standard packages
+# Standard moduless
 import re
 import sys
 from collections import OrderedDict
 
-# Installed packages
+# Installed moduless
+# pylint: disable=import-error
 # TODO: install kera dynamically???
 ## TEST: sys.stderr.write("here\n")
 ## BAD:
@@ -61,10 +62,11 @@ from sklearn.preprocessing import LabelEncoder
 ## OLD: import tensorflow as tf
 ## TEST: tf = None
 
-# Local packages
+# Local moduless
 from mezcla import debug
 from mezcla import glue_helpers as gh
 from mezcla import system
+round3 = system.round3
 from mezcla import text_utils
 from mezcla.text_utils import getenv_ints
 
@@ -76,7 +78,7 @@ assert((sys.version_info.major >= 3) and (sys.version_info.minor >= 6))
 
 DEFAULT_VERBOSITY = ((debug.get_level() + 1) // 2)
 VERBOSITY_LEVEL = system.getenv_int("VERBOSITY_LEVEL", DEFAULT_VERBOSITY)
-DATA_FILE = system.getenv_text("DATA_FILE", "samples/pima-indians-diabetes.csv")
+DATA_FILE = system.getenv_text("DATA_FILE", "examples/pima-indians-diabetes.csv")
 FIELD_SEP = system.getenv_text("FIELD_SEP", ",")
 BRUTE_FORCE = system.getenv_bool("BRUTE_FORCE", False)
 RANDOM_OPTIMIZATION = (not BRUTE_FORCE)
@@ -117,10 +119,6 @@ DEFAULT_HIDDEN_UNITS = getenv_ints("HIDDEN_UNITS", "20 30")
 #...............................................................................
 # Utility functions
 
-def round3(num):
-    """Round NUM using precision of 3"""
-    return system.round_num(num, 3)
-
 def non_negative(num):
     """Whether integer NUM > -1"""
     return (num > -1)
@@ -158,7 +156,8 @@ def create_keras_model(num_input_features=None, num_classes=None, hidden_units=N
         # note: -1 needs to be specified in the estimator constructor in order for the grid search
         # variable to be recognized, therefore only non-negative counts are considered.
         hidden_unit_counts = [(kwargs.get(v) or -1) for v in HIDDEN_UNIT_VARS]
-        num_non_negative = sum([int(non_negative(n)) for n in hidden_unit_counts])
+        ## OLD: num_non_negative = sum([int(non_negative(n)) for n in hidden_unit_counts])
+        num_non_negative = sum(int(non_negative(n)) for n in hidden_unit_counts)
         debug.trace_fmt(5, "hidden_unit_counts={huc} num_non_negative={nnn}", huc=hidden_unit_counts, nnn=num_non_negative)
         if (num_non_negative > 0):
             hidden_units = hidden_unit_counts
@@ -204,6 +203,7 @@ class MyKerasClassifier(KerasClassifier):
         super().__init__(**kwargs)
 
     def check_params(self, params):
+        """Make sure all params supported"""
         ok = (not system.difference(list(params.keys()), self.params))
         debug.trace_fmt(6, "{cl}.check_params({p}) => {r}", cl=self.class_name, p=params, r=ok)
         return ok
@@ -283,6 +283,7 @@ def main():
 
     # Create initial model
     dummy_hidden_unit_params = {v:-1 for v in HIDDEN_UNIT_VARS}
+    # pylint: disable=unnecessary-lambda-assignment
     create_model_fn = lambda: create_keras_model(num_input_features=num_features,
                                                  num_classes=num_categories)
     ## OLD: model = KerasClassifier(build_fn=create_model_fn, verbose=VERBOSITY_LEVEL)
