@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Convenience class for regex searching, providing simple wrapper around
 # static match results.
@@ -12,12 +12,19 @@
 #    ...
 #    if (my_re.search(r"^(\d+)\:?(\d*)\s+(\d+)\:?(\d*)\s+\S.*", line)):
 #        (start_hours, start_mins, end_hours, end_mins) = my_re.groups()
+#................................................................................
+# Regex cheatsheet:
+#     (?:regex)               non-capturing group
+#     (?<!regex)              negative lookbehind
+#     (?!regex)               negative lookahead
+#     (?=regex)               positive lookahead
+#     *?  and  +?             non-greedy match
 #
 #--------------------------------------------------------------------------------
 # TODO:
+# - Flesh out cheatsheet
 # - Add examples for group(), groups(), etc.
 # - Clean up script (e.g., regex => regex_wrapper).
-# - Add perl-inspired accessors (e.g., PREMATCH, POSTMATCH).
 #
 
 """Wrapper class for regex match results"""
@@ -27,11 +34,12 @@ import re
 ## TODO: from re import *
 
 # Installed packages
-import six
+## OLD: import six
 
 # Local packages
 from mezcla import debug
 from mezcla import system
+## DEBUG: debug.trace(4, "my_regex: {__file__}")
 
 # Expose public symbols from re package, plus the wrapper class and global instance
 ## OLD:
@@ -48,9 +56,20 @@ if RE_ALL:
 else:
     debug.trace(4, "Omitting use of re __all__")
 
-REGEX_TRACE_LEVEL = system.getenv_int("REGEX_TRACE_LEVEL", debug.QUITE_DETAILED,
-                                      "Trace level for my_regex")
-    
+# Environment options
+## OLD
+## REGEX_TRACE_LEVEL = system.getenv_int("REGEX_TRACE_LEVEL", debug.QUITE_DETAILED,
+##                                       "Trace level for my_regex")
+REGEX_DEBUG_LEVEL = system.getenv_int(
+    "REGEX_DEBUG_LEVEL", debug.QUITE_DETAILED,
+    desc="Alias for REGEX_TRACE_LEVEL")
+REGEX_TRACE_LEVEL = system.getenv_int(
+    "REGEX_TRACE_LEVEL", REGEX_DEBUG_LEVEL,
+    desc="Trace level for my_regex")
+REGEX_WARNINGS = system.getenv_bool(
+    "REGEX_WARNINGS", debug.debugging(debug.USUAL),
+    desc="Include warnings about regex's such as f-string")
+
 ## TODO # HACK: make sure regex can be used as plug-in replacement 
 ## from from re import *
 
@@ -104,10 +123,12 @@ class regex_wrapper():
     def __init__(self, ):
         debug.trace_fmtd(4, "my_regex.__init__(): self={s}", s=self)
         self.match_result = None
+        self.search_text = None
         # TODO: self.regex = ""
 
         # HACK: Import attributes from re class
-        # TODO: see if clean way to do this
+        # TODO3: see if clean way to do this
+        # TODO4: find way to disable pylint no-member warning
         try:
             for var in re.__all__:
                 if var not in dir(self):
@@ -118,11 +139,28 @@ class regex_wrapper():
 
     def check_pattern(self, regex):
         """Apply sanity checks to REGEX when debugging
-        Note: Added to account for potential f-string confusion"""
-        # TODO: Add way to disable check
+        Note: Added to account for potential missing f-string prefix"""
+        debug.trace(self.TRACE_LEVEL + 1, f"check_pattern({regex})")
         debug.reference_var(self)
-        if (debug.debugging(1) and re.search(r"[^{]\{[A-Fa-f0-9][^{}]+\}[^}]", regex)):
-            system.print_error(f"Warning: potentially unresolved f-string in {regex}")
+        # note: checks for variable reference in braces (e.g., "Hi, {name}!")
+        ## BAD: check_regex = r"([^{]|^)\{[^0-9][A-Fa-f0-9]*[^{}]+\}([^}]|$)"
+        ## ALT: check_regex = r"([^\{]|^)\{([[A-Z][A-Z0-9]*[^\{\}]+)\}([^\}]|$)"
+        check_regex = r"([^{]|^){[A-Z][A-Z0-9]*[^{}]+}([^}]|$)"
+        if isinstance(regex, bytes):
+            ## OLD: regex = regex.encode()
+            check_regex = check_regex.encode()
+        if REGEX_WARNINGS:
+            debug.trace_expr(self.TRACE_LEVEL + 2, check_regex, delim="\n")
+            match = re.search(check_regex, regex, flags=re.IGNORECASE)
+            if match:
+                # Ignore regex operators within f-string replacement
+                re_operator_pattern = r"[\*\+\?]"
+                if isinstance(match.string, bytes):
+                    re_operator_pattern = re_operator_pattern.encode()
+                if re.search(re_operator_pattern, match.string):
+                    match = None
+            if match:
+                system.print_error(f"Warning: potentially unresolved f-string in {regex!r} at {match.start(0)}")
 
     def search(self, regex, text, flags=0, base_trace_level=None):
         """Search for REGEX in TEXT with optional FLAGS and BASE_TRACE_LEVEL (e.g., 6)"""
@@ -130,12 +168,15 @@ class regex_wrapper():
         if base_trace_level is None:
             base_trace_level = self.TRACE_LEVEL
         debug.trace_fmtd((1 + base_trace_level), "my_regex.search({r!r}, {t!r}, {f}): self={s}",
-                         r=regex, t=text, f=flags, s=self)
-        debug.assertion(isinstance(text, six.string_types))
+                         r=regex, t=text, f=flags, s=self, max_len=2048)
+        ## OLD: debug.assertion(isinstance(text, six.string_types))
+        debug.assertion(isinstance(text, (str, bytes)) and (isinstance(regex, type(text))))
+        self.search_text = text
         self.check_pattern(regex)
         self.match_result = re.search(regex, text, flags)
         if self.match_result:
-            debug.trace_fmt(base_trace_level, "match: {m!r}; regex: {r}", m=self.grouping(), r=regex)
+            debug.trace_fmt(base_trace_level, "match: {m!r}; regex: {r!r}", m=self.grouping(), r=regex)
+            debug.trace_object(base_trace_level + 1, self.match_result)
         return self.match_result
 
     def match(self, regex, text, flags=0, base_trace_level=None):
@@ -144,17 +185,19 @@ class regex_wrapper():
         if base_trace_level is None:
             base_trace_level = self.TRACE_LEVEL
         debug.trace_fmtd((1 + base_trace_level), "my_regex.match({r!r}, {t!r}, {f}): self={s}",
-                         r=regex, t=text, f=flags, s=self)
+                         r=regex, t=text, f=flags, s=self, max_len=2048)
+        self.search_text = text
         self.check_pattern(regex)
         self.match_result = re.match(regex, text, flags)
         if self.match_result:
             debug.trace_fmt(base_trace_level, "match: {m!r}; regex: {r!r}", m=self.grouping(), r=regex)
+            debug.trace_object(base_trace_level + 1, self.match_result)
         return self.match_result
 
     def get_match(self):
         """Return match result object for last search or match"""
         result = self.match_result
-        debug.trace_fmtd(self.TRACE_LEVEL, "my_regex.get_match() => {r}: self={s}",
+        debug.trace_fmtd(self.TRACE_LEVEL, "my_regex.get_match() => {r!r}: self={s}",
                          r=result, s=self)
         return result
 
@@ -162,7 +205,7 @@ class regex_wrapper():
         """Return group NUM from match result from last search"""
         debug.assertion(self.match_result)
         result = self.match_result and self.match_result.group(num)
-        debug.trace_fmtd(self.TRACE_LEVEL, "my_regex.group({n}) => {r}: self={s}",
+        debug.trace_fmtd(self.TRACE_LEVEL, "my_regex.group({n}) => {r!r}: self={s}",
                          n=num, r=result, s=self)
         return result
 
@@ -170,7 +213,7 @@ class regex_wrapper():
         """Return all groups in match result from last search"""
         debug.assertion(self.match_result)
         result = self.match_result and self.match_result.groups()
-        debug.trace_fmt(self.TRACE_LEVEL, "my_regex.groups() => {r}: self={s}",
+        debug.trace_fmt(self.TRACE_LEVEL, "my_regex.groups() => {r!r}: self={s}",
                         r=result, s=self)
         return result
 
@@ -178,19 +221,19 @@ class regex_wrapper():
         """Return groups for match result or entire matching string if no groups defined"""
         # Note: this is intended to facilitate debug tracing; see example in search method above
         result = self.match_result and (self.match_result.groups() or self.match_result.group(0))
-        debug.trace_fmt(self.TRACE_LEVEL + 1, "my_regex.grouping() => {r}: self={s}", r=result, s=self)
+        debug.trace_fmt(self.TRACE_LEVEL + 1, "my_regex.grouping() => {r!r}: self={s}", r=result, s=self)
         return result
 
     def start(self, group=0):
         """Start index for GROUP"""
         result = self.match_result and self.match_result.start(group)
-        debug.trace_fmt(self.TRACE_LEVEL + 1, "my_regex.start({g}) => {r}: self={s}", r=result, s=self, g=group)
+        debug.trace_fmt(self.TRACE_LEVEL + 1, "my_regex.start({g}) => {r!r}: self={s}", r=result, s=self, g=group)
         return result
 
     def end(self, group=0):
         """End index for GROUP"""
         result = self.match_result and self.match_result.end(group)
-        debug.trace_fmt(self.TRACE_LEVEL + 1, "my_regex.end({g}) => {r}: self={s}", r=result, s=self, g=group)
+        debug.trace_fmt(self.TRACE_LEVEL + 1, "my_regex.end({g}) => {r!r}: self={s}", r=result, s=self, g=group)
         return result
 
     def sub(self, pattern, replacement, string, *, count=0, flags=0):
@@ -209,15 +252,42 @@ class regex_wrapper():
     def split(self, pattern, string, maxsplit=0, flags=0):
         """Use PATTERN to split STRING, optionally up to MAXSPLIT with FLAGS"""
         result = re.split(pattern, string, maxsplit, flags)
-        debug.trace_fmt(self.TRACE_LEVEL, "split{args} => {r}",
-                        args=tuple([pattern, string, maxsplit, flags]), r=result)
+        debug.trace_fmt(self.TRACE_LEVEL, "split{args} => {r!r}",
+                        args=tuple([pattern, string, maxsplit, flags]), r=result, max_len=2048)
         return result
     
     def findall(self, pattern, string, flags=0):
         """Use PATTERN to split STRING, optionally with specified FLAGS"""
         result = re.findall(pattern, string, flags)
-        debug.trace_fmt(self.TRACE_LEVEL, "findall{args} => {r}",
-                        args=tuple([pattern, string, flags]), r=result)
+        debug.trace_fmt(self.TRACE_LEVEL, "findall{args} => {r!r}",
+                        args=tuple([pattern, string, flags]), r=result, max_len=2048)
+        return result
+
+    def escape(self, text):
+        """Escape special characters in TEXT"""
+        ## TODO3: make static method
+        result = re.escape(text)
+        debug.trace(self.TRACE_LEVEL + 1, f"escape({text!r}) => {result!r}")
+        return result
+
+    def pre_match(self):
+        """Text preceding the match or None if not defined"""
+        result = None
+        if self.match_result:
+            start = 0
+            end = self.match_result.span(0)[0]
+            result = self.search_text[start: end]
+        debug.trace(self.TRACE_LEVEL, f"pre_match() => {result!r}")
+        return result
+    
+    def post_match(self):
+        """Text following the match or None if not defined"""
+        result = None
+        if self.match_result:
+            start = self.match_result.span(0)[1]
+            end = len(self.search_text)
+            result = self.search_text[start: end]
+        debug.trace(self.TRACE_LEVEL, f"post_match() => {result!r}")
         return result
     
 #...............................................................................
