@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Runs training data through a variety of supervised classifiers, as an
 # illustration on how to use pandas and sklearn to do machine learning.
@@ -159,6 +159,7 @@ DUMP_MODEL = getenv_bool("DUMP_MODEL", False,
 # Globals
 devel_classifiers = [DEVEL_CLASSIFIER]
 validation_classifiers = [VALIDATION_CLASSIFIER]
+plt = None
 
 #...............................................................................
 # Optional packages
@@ -263,6 +264,7 @@ def main():
     # Reorder columns so classication variable last
     if (CLASS_VAR and (CLASS_VAR != dataset.columns[-1])):
         debug.trace_fmtd(4, "Re-arranging columns")
+        # pylint: disable=no-member
         columns = system.difference(list(dataset.columns), CLASS_VAR)
         columns.append(CLASS_VAR)
         dataset = dataset[columns]
@@ -325,6 +327,7 @@ def main():
         if NUMERIC_CLASSES:
             y = y.astype(np.float64)
     else:
+        # pylint: disable=no-member
         array = dataset.values
         debug.trace_object(7, array, "array")
         num_features = (array.shape[1] - 1)
@@ -391,6 +394,7 @@ def main():
         debug.trace_fmt(4, 'misc_xgb_params={m}', m=misc_xgb_params)
         models.append((XGB_CLASSIFIER, xgb.XGBClassifier(**misc_xgb_params)))
     if INCLUDE_KERAS:
+        # pylint: disable=unnecessary-lambda-assignment
         create_model_fn = lambda: create_keras_model(num_input_features=num_features,
                                                      num_classes=num_classes)
         models.append((KERAS_CLASSIFIER,
