@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Test(s) for gensim_test.py
 #
@@ -22,22 +22,39 @@ import re
 
 # Installed packages
 import pytest
+## TAKE1
+## # note: The gensim module is not installed by default, so tests skipped if not found
+## try:
+##     import gensim
+## except:
+##     gensim = None
 
 # Local packages
 from mezcla import debug
 from mezcla import glue_helpers as gh
 from mezcla import tpo_common as tpo
-from mezcla.unittest_wrapper import TestWrapper
+from mezcla.unittest_wrapper import TestWrapper, invoke_tests
+from mezcla import system
 
 # Note: Two references are used for the module to be tested:
-#    THE_MODULE:	    global module object
-import mezcla.gensim_test as THE_MODULE
+#    THE_MODULE:            global module object
+try:
+    ## OLD: # pylint: disable=ungrouped-imports
+    import gensim
+    import mezcla.gensim_test as THE_MODULE
+except:
+    ## TEST (maldito pytest):
+    ## print("hey, hey, hey!")
+    debug.trace_exception(3, "importing gensim_test")
+    gensim = None
+    THE_MODULE = None
 
 class TestGensimTest(TestWrapper):
-    """Class for testcase definition"""
+    """Class for script-level testcase definition"""
     script_file = TestWrapper.get_module_file_path(__file__)
-    script_module = TestWrapper.derive_tested_module_name(__file__)
+    script_module = TestWrapper.get_testing_module_name(__file__, THE_MODULE)
 
+    @pytest.mark.skipif(not gensim, reason="gensim module missing")
     def test_data_file(self):
         """Tests results over a known data file (LICENSE.txt)"""
         tpo.debug_print("test_data_file()", 4)
@@ -48,20 +65,34 @@ class TestGensimTest(TestWrapper):
         ## TODO: assert re.search("storing corpus in Matrix Market format", output)
         assert gh.non_empty_file(temp_data_file.replace(".txt", ".bow.mm"))
         debug.trace_expr(5, output)
+        ## TODO3: check for expected entries (e.g., .wordids.txt.bz2 file)
         return
 
+    @pytest.mark.skipif(not gensim, reason="gensim module missing")
     def test_vector_printing(self):
         """Test printing of corpus vector for simple input"""
         tpo.debug_print("test_vector_printing()", 4)
         temp_file = self.temp_base + ".txt"
-        gh.write_file(temp_file, "My dog has fleas.\n")
+        system.write_file(temp_file, "My dog has fleas.\n")
         output = self.run_script("--print --verbose", temp_file)
         assert re.search(r"\(u?'dog', 1\),.*\(u?'has', 1\)", output)
+        ## TODO4: likewise do non-trivial input like LICENSE.txt
         debug.trace_expr(5, output)
         return
 
+    
+class TestGensimTest2:
+     """Class for internal testcase definitions"""
+
+     @pytest.mark.skipif(not gensim, reason="gensim module missing")
+     def test_corpus_data(self):
+         """Test creation of corpus from a file"""
+         corpus = THE_MODULE.CorpusData(__file__)
+         # note: currently 81 unique tokens extracted
+         assert len(list(corpus)) > 50
+   
 #------------------------------------------------------------------------
 
 if __name__ == '__main__':
     debug.trace_current_context()
-    pytest.main([__file__])
+    invoke_tests(__file__)
