@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Test(s) for ../text_utils.py
 #
@@ -12,71 +12,40 @@
 """Tests for text_utils module"""
 
 # Standard packages
-import re
+## OLD: import re
 import os
 
 # Installed packages
-import pytest
+## OLD: import pytest
 
 # Local packages
 from mezcla import debug
 from mezcla import glue_helpers as gh
-from mezcla import system
+## OLD: from mezcla import system
+from mezcla.unittest_wrapper import TestWrapper, invoke_tests
+from mezcla.tests.common_module import normalize_text
 
 # Note: Two references are used for the module to be tested:
-#    THE_MODULE:	    global module object
+#    THE_MODULE:            global module object
 import mezcla.text_utils as THE_MODULE
 
-HTML_FILENAME = "./resources/simple-window-dimensions.html"
-
-EXPECTED_TEXT = """
-   Simple window dimensions
-
-   Simple window dimensions
-
-      Legend:
-        Screen dimensions:    ???
-        Browser dimensions:   ???
-
-   JavaScript is required
-"""
-#
-# NOTE: Whitespace and punctuation gets normalized
-# TODO: restore bullet points (e.g., "* Screen dimensions")
-
 MS_WORD_FILENAME = "./resources/spanish-accents.docx"
 
 MS_WORD_TEXT = "Tío Tomás\t\t\t\tUncle Tom\n\n¡Buenos días!\t\t\t\tGood morning\n\nçãêâôöèäàÃëÇÂîòïÔìðÊÅåùÀŠý\t\tcaeaooeaaAeCAioiOioEAauASy"
 
 
-def normalize_text(text):
-    """Trim excess whitespace and convert punctuation to <PUNCT>"""
-    # EX: normalize_test_text("   h  e y?! ") => "h e y<PUNCT>"
-    result = text.strip()
-    result = re.sub(r"\s+", " ", result)
-    result = re.sub(r"[^\w\s]+", "<PUNCT>", result)
-    debug.trace(4, f"normalize_test_text({text}) => {result}")
-    return result
-
-
-class TestTextUtils:
+class TestTextUtils(TestWrapper):
     """Class for test case definitions"""
+    # note: script_module used in argument parsing sanity check (e.g., --help)
+    script_module = TestWrapper.get_testing_module_name(__file__, THE_MODULE)
 
-    def test_init_BeautifulSoup(self):
-        """Ensure init_BeautifulSoup works as expected"""
-        debug.trace(4, "test_init_BeautifulSoup()")
-        THE_MODULE.BeautifulSoup = None
-        THE_MODULE.init_BeautifulSoup()
-        assert THE_MODULE.BeautifulSoup is not None
-
-    def test_html_to_text(self):
-        """Ensure html_to_text works as expected"""
-        # TODO: move into test_html_utils.py
-        debug.trace(4, "test_html_to_text()")
-        html_path = gh.resolve_path(HTML_FILENAME)
-        html = system.read_file(html_path)
-        text = THE_MODULE.html_to_text(html)
-        assert normalize_text(text) == normalize_text(EXPECTED_TEXT)
+    ## OLD:
+    ## def test_init_BeautifulSoup(self):
+    ##     """Ensure init_BeautifulSoup works as expected"""
+    ##     debug.trace(4, "test_init_BeautifulSoup()")
+    ##     THE_MODULE.BeautifulSoup = None
+    ##     THE_MODULE.init_BeautifulSoup()
+    ##     assert THE_MODULE.BeautifulSoup is not None
 
     def test_init_textract(self):
         """Ensure init_textract works as expected"""
@@ -92,33 +61,34 @@ class TestTextUtils:
         text = THE_MODULE.document_to_text(doc_path)
         assert normalize_text(text) == normalize_text(MS_WORD_TEXT)
 
-    def test_extract_html_images(self):
-        """Ensure extract_html_images works as expected"""
-        debug.trace(4, "test_extract_html_images()")
-
-        url = 'example.com/'
-        html = (
-            '<!DOCTYPE html>\n'
-            '<html>\n'
-            '<body>\n'
-            '<h2>The target Attribute</h2>\n'
-            '<div class="some-class">this is a div</div>\n'
-            '<div class="some-class another-class">'
-            '<img src="smiley.gif" alt="Smiley face" width="42" height="42" style="border:5px solid black">\n'
-            '<img src="some_image.jpg" alt="Some image" width="42" height="42" style="border:5px solid black">\n'
-            '<img src="hidden.jpg" alt="this is a hidden image" width="42" height="42" style="display:none">\n'
-            '</div>'
-            '</body>\n'
-            '</html>\n'
-        )
-        images_urls = [
-            f'{url}/smiley.gif',
-            f'{url}/some_image.jpg'
-        ]
-
-        result = THE_MODULE.extract_html_images(html, url)
-        assert result == images_urls
-        assert 'hidden.jpg' not in images_urls
+    ## OLD:
+    ## def test_extract_html_images(self):
+    ##     """Ensure extract_html_images works as expected"""
+    ##     debug.trace(4, "test_extract_html_images()")
+    ##
+    ##     url = 'example.com'
+    ##     html = (
+    ##         '<!DOCTYPE html>\n'
+    ##         '<html>\n'
+    ##         '<body>\n'
+    ##         '<h2>The target Attribute</h2>\n'
+    ##         '<div class="some-class">this is a div</div>\n'
+    ##         '<div class="some-class another-class">'
+    ##         '<img src="smiley.gif" alt="Smiley face" width="42" height="42" style="border:5px solid black">\n'
+    ##         '<img src="some_image.jpg" alt="Some image" width="42" height="42" style="border:5px solid black">\n'
+    ##         '<img src="hidden.jpg" alt="this is a hidden image" width="42" height="42" style="display:none">\n'
+    ##         '</div>'
+    ##         '</body>\n'
+    ##         '</html>\n'
+    ##     )
+    ##     images_urls = [
+    ##         f'{url}/smiley.gif',
+    ##         f'{url}/some_image.jpg'
+    ##     ]
+    ##
+    ##     result = THE_MODULE.extract_html_images(html, url)
+    ##     assert result == images_urls
+    ##     assert 'hidden.jpg' not in images_urls
 
     def test_version_to_number(self):
         """Ensure version_to_number works as expected"""
@@ -163,10 +133,11 @@ class TestTextUtils:
         assert THE_MODULE.extract_int_list("1   2  foobar", default_value=9) == [1, 2, 9]
         assert THE_MODULE.extract_int_list("1   2  3.45", default_value=230) == [1, 2, 230]
 
-    def test_getenv_ints(self, monkeypatch):
+    def test_getenv_ints(self):
         """Ensure getenv_ints works as expected"""
         debug.trace(4, "test_getenv_ints()")
-        monkeypatch.setenv("DUMMY-VARIABLE", "0, 1  2  3 4 ", prepend=os.pathsep)
+        ## INFO: help(self.monkeypatch)
+        self.monkeypatch.setenv("DUMMY-VARIABLE", "0, 1  2  3 4 ", prepend=os.pathsep)
         assert THE_MODULE.getenv_ints("DUMMY-VARIABLE", str(list(range(5)))) == [0, 1, 2, 3, 4]
 
     def test_is_symbolic(self):
@@ -178,6 +149,12 @@ class TestTextUtils:
         assert not THE_MODULE.is_symbolic(123)
         assert not THE_MODULE.is_symbolic(0.1)
 
+    def test_is_numeric(self):
+        """Ensure is_numeric works as expected"""
+        debug.trace(4, "test_is_numeric()")
+        assert THE_MODULE.is_numeric("456")
+        assert not THE_MODULE.is_numeric("three")
+        
     def test_make_fixed_length(self):
         """Ensure make_fixed_length works as expected"""
         debug.trace(4, "make_fixed_length()")
@@ -188,4 +165,4 @@ class TestTextUtils:
 
 if __name__ == '__main__':
     debug.trace_current_context()
-    pytest.main([__file__])
+    invoke_tests(__file__)
