@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Test(s) for ../xml_utils.py
 #
@@ -33,27 +33,16 @@ import re
 import pytest
 
 # Local packages
-from mezcla.unittest_wrapper import TestWrapper
+from mezcla.unittest_wrapper import TestWrapper, invoke_tests
 from mezcla import debug
-## TODO: from mezcla import system
 from mezcla import glue_helpers as gh
+from mezcla import system
 
 # Note: Two references are used for the module to be tested:
-#    THE_MODULE:	    global module object
+#    THE_MODULE:            global module object
 import mezcla.xml_utils as THE_MODULE
-#
-# Note: sanity test for customization (TODO: remove if desired)
-if not re.search(__file__, r"\btemplate.py$"):
-    debug.assertion("mezcla.template" not in str(THE_MODULE))
-
-## TODO:
-## # Environment options
-## # Note: These are just intended for internal options, not for end users.
-## # It also allows for enabling options in one place.
-## #
-## FUBAR = system.getenv_bool("FUBAR", False,
-##                            description="Fouled Up Beyond All Recognition processing")
 
+# Constants
 NESTED_XML = """<?xml version="1.1"?>
 <xml><a><b>1<c>2<d/>3</c></b>4</a></xml>
 """
@@ -61,13 +50,15 @@ NESTED_XML = """<?xml version="1.1"?>
 class TestXmlUtils(TestWrapper):
     """Class for testcase definition"""
     script_file = TestWrapper.get_module_file_path(__file__)
-    script_module = TestWrapper.get_testing_module_name(__file__)
+    script_module = TestWrapper.get_testing_module_name(__file__, THE_MODULE)
 
+
+    @pytest.mark.xfail                   # TODO: remove xfail
     def test_data_file(self):
         """Makes sure simple XML data file parsed OK"""
         debug.trace(4, "TestXmlUtils.test_data_file()")
         data = "<xml><a>A</a><b>B</b></xml>"
-        gh.write_file(self.temp_file, data)
+        system.write_file(self.temp_file, data)
         output = self.run_script("", self.temp_file)
         assert "a: A" in output
         assert "b: B" in output
@@ -85,4 +76,4 @@ class TestXmlUtils(TestWrapper):
 
 if __name__ == '__main__':
     debug.trace_current_context()
-    pytest.main([__file__])
+    invoke_tests(__file__)
