@@ -2,7 +2,7 @@
 """
 Package for Term Frequency (TF) Inversion Document Frequency (IDF): TF-IDF
 """
-__version__ = "1.4"
+__version__ = "1.7"
 
 ## TODO
 ## # HACK: For relative imports to work in Python 3.6
@@ -10,3 +10,7 @@ __version__ = "1.4"
 ## import os
 ## import sys;
 ## sys.path.append(os.path.dirname(os.path.abspath(__file__)))
+
+from mezcla.tfidf.corpus import MIN_NGRAM_SIZE, MAX_NGRAM_SIZE
+
+__all__ = ["MIN_NGRAM_SIZE", "MAX_NGRAM_SIZE"]
