@@ -3,37 +3,36 @@
 """A document object used for the "term frequency" (TF) in TF-IDF.
 
 Example:
-    >>> from mezcla.tfidf.document import Document
-    >>> from mezcla.tfidf.preprocess import Preprocessor
-    >>> input_text = '''Mary had a little lamb, his fur was white as snow
-            Everywhere the child went, the lamb, the lamb was sure to go'''
-    >>> d = Document(input_text, Preprocessor(stemmer=lambda x: x))
-    >>> d__gramsize 
+    >>> import mezcla.tfidf.document as mtd
+    >>> import mezcla.tfidf.preprocess as mtp
+    >>> input_text = 'Mary had a little lamb, his fur was white as snow. Everywhere the child went, the lamb, the lamb was sure to go'
+    >>> d = mtd.Document(input_text, mtp.Preprocessor(stemmer=lambda x: x))
+    >>> d.preprocessor.gramsize 
     1
-    >>> d.tf_freq('lamb')
+    >>> round(d.tf_freq('lamb'))
     3
-    >>> d.tf_raw('lamb')
-    0.04411764705882353
-
+    >>> round(d.tf_raw('lamb'), 3)
+    0.044
 """
 # HACK: Attempt at fixing up a very stale example!
 # TODO: Make sure the tf_raw calculation revision is correct.
 # TODO1: revise docstrings wrt preferred keywords (i.e., not deprecated)
+# TODO1: drop Preprocessor and put into constructor as default if None
+# NOTE: examples use import...as for sake of reload
 
 from __future__ import absolute_import, division
 
 # Standard packages
+from collections import defaultdict
 import math
-## OLD: import os
 import random
-## OLD: import sys
 
 # Installed packages
 from mezcla import debug
 from mezcla import system
 
 # Local packages
-## OLD: from .preprocess import clean_text
+from mezcla.tfidf.config import BASE_DEBUG_LEVEL as BDL
 from mezcla.tfidf.preprocess import clean_text, Preprocessor
 
 # TPO: environment option for weight singleton occurrences low
@@ -50,7 +49,7 @@ class Document(object):
         text (list): cleaned text, set on init
     """
 
-    def __init__(self, raw_text, preprocessor):
+    def __init__(self, raw_text, preprocessor=None):
         """All you need is the text body and gramsize (number words in ngram).
 
         raw_text
@@ -58,11 +57,14 @@ class Document(object):
         preprocessor
             initalized instance of a preprocessor
         """
+        ## TODO2: fix gramsize reference (in preprocessor)
         self.id = None
         self.text = clean_text(raw_text)
         self.__keywordset = None
         self.__max_raw_frequency = None
         self.__length = None
+        if (preprocessor is None):
+            preprocessor = Preprocessor(stemmer=lambda x: x)
         self.preprocessor = preprocessor
 
     def __contains__(self, ngram):
@@ -77,7 +79,7 @@ class Document(object):
         """The length of the document is the number of ngrams."""
         # TODO: rename to __size__??
         if not self.__length:
-            self.__length = sum([len(x) for x in self.keywordset])
+            self.__length = sum(len(x) for x in self.keywordset.values())
         return self.__length
 
     @property
@@ -86,7 +88,7 @@ class Document(object):
         ## if not self.__max_raw_frequency:
         if self.__max_raw_frequency is None:
             biggest_kw = ''
-            for kw in self.keywordset:
+            for kw in self.keywordset.values():
                 if len(kw) > len(biggest_kw):
                     biggest_kw = kw
             self.__max_raw_frequency = len(biggest_kw)
@@ -97,16 +99,14 @@ class Document(object):
         """Important for fast check if ngram in document.
         Note: Converts self.keywordset hash to set proper.
         """
-        ## NOTE: 
-        ## OLD: return set([x for x in self.keywordset])
-        ## OLD: return {x for x in self.keywordset}
         return set(self.keywordset)
 
     @property
     def keywordset(self):
         """Return a set of keywords in the document with all their locations."""
         if not self.__keywordset:
-            self.__keywordset = {}
+            ## OLD: self.__keywordset = {}
+            self.__keywordset = defaultdict(str)
             for kw in self.keywords:
                 if kw.text not in self.__keywordset:
                     self.__keywordset[kw.text] = kw
@@ -122,12 +122,14 @@ class Document(object):
         if (num_occurrences == 1) and PENALIZE_SINGLETONS:
             num_occurrences = 0
         tf_raw = float(num_occurrences) / len(self)
-        debug.trace_fmt(6, "tf_raw({ng}): num_occ={no} len(self)={l} result={r}",
+        debug.trace_fmt(BDL + 1, "tf_raw({ng}): num_occ={no} len(self)={l} result={r}",
                         ng=ngram, no=num_occurrences, l=len(self), r=tf_raw)
         return tf_raw
 
     def tf_log(self, ngram):
-        """The log frequency of an ngram in a document."""
+        """The log (relative) frequency of an ngram in a document.
+        Note: uses add-1 smoothing
+        """
         # TODO: is this formula right? Wikipedia is often wrong...
         return 1 + math.log(self.tf_raw(ngram))
 
@@ -145,7 +147,7 @@ class Document(object):
     def tf_freq(self, ngram):
         """Returns frequency count for NGRAM"""
         num_occurrences = len(self[ngram]) if ngram in self else 0
-        debug.trace_fmt(6, "tf_freq({ng}): num_occ={no} len(self)={l} result={r}",
+        debug.trace_fmt(BDL + 1, "tf_freq({ng}): num_occ={no} len(self)={l} result={r}",
                         ng=ngram, no=num_occurrences, l=len(self), r=num_occurrences)
         return num_occurrences
     
@@ -158,32 +160,24 @@ class Document(object):
             tf_weight = 'basic'
         if normalize:
             ## TPO: fix referernce to normalize_term
-            ## BAD: ngram = self.normalize_term(ngram)
             ngram = self.preprocessor.normalize_term(ngram)
-        ## OLD
-        ## if tf_weight == 'log':
-        ##     return self.tf_log(ngram)
-        ## elif tf_weight == 'norm_50':
-        ##     return self.tf_norm_50(ngram)
-        ## elif tf_weight == 'binary':
-        ##     return self.tf_binary(ngram)
-        ## elif tf_weight == 'basic':
-        ##     return self.tf_raw(ngram)
-        ## # TPO: allow for raw frequency
-        ## elif tf_weight == 'freq':
-        ##     return self.tf_freq(ngram)
+        result = 0
         if tf_weight == 'log':
-            return self.tf_log(ngram)
-        if tf_weight == 'norm_50':
-            return self.tf_norm_50(ngram)
-        if tf_weight == 'binary':
-            return self.tf_binary(ngram)
-        if tf_weight == 'basic':
-            return self.tf_raw(ngram)
+            result = self.tf_log(ngram)
+        elif tf_weight == 'norm_50':
+            result = self.tf_norm_50(ngram)
+        elif tf_weight == 'binary':
+            result = self.tf_binary(ngram)
+        elif tf_weight == 'basic':
+            result = self.tf_raw(ngram)
         # TPO: allow for raw frequency
-        if tf_weight == 'freq':
-            return self.tf_freq(ngram)
-        raise ValueError("Invalid tf_weight: " + tf_weight)
+        elif tf_weight == 'freq':
+            result = self.tf_freq(ngram)
+        else:
+            raise ValueError("Invalid tf_weight: " + tf_weight)
+        debug.trace_fmt(BDL + 2, "tf({ng}, tfw={tfw}, norm={n}) => {r}\n",
+                        ng=ngram, tfw=tf_weight, n=normalize, r=result)
+        return result
 
     def randgram(self):
         """Return a random gram. Limited to first 100 ngrams."""
@@ -210,11 +204,11 @@ def main():
     """Entry point for script: just runs a simple test"""
     text = "my man fran is not a man"
     d = Document(text, Preprocessor(gramsize=1, stemmer=lambda x: x))
-    debug.trace_expr(4, [d.tf_freq(t) for t in text.split()])
+    debug.trace_expr(BDL - 1, [d.tf_freq(t) for t in text.split()])
     debug.assertion(d.tf_freq("man") > d.tf_freq("fran"))
     
 #-------------------------------------------------------------------------------
     
 if __name__ == '__main__':
-    system.print_stderr(f"Warning: {__file__} is not intended to be run standalone. A simple test willl be run.")
+    system.print_stderr(f"Warning: {__file__} is not intended to be run standalone. A simple test will be run.")
     main()
