@@ -3,16 +3,18 @@
 # TODO:
 # - Fix env-spec above and put coding on same line.
 # - Verify the fixup for PEP-0479 didn't break anything in Python 2 (e.g., for regression testing).
+# - *** Remove Python2-specific code!
 #
 
 """Pre-processing step for text.
 
 Example:
-    >>> from mezcla.tfidf.preprocess import Preprocessor
-    >>> p = Preprocessor(gramsize=2)
-    >>> sorted(list([k.text for k in p.yield_keywords("a b c")]))
-    ["a b", "b c"]
+    >>> import mezcla.tfidf.preprocess as mtp
+    >>> p = mtp.Preprocessor(gramsize=2)
+    >>> sorted(list([k.text for k in p.yield_keywords('a b c')]))
+    ['a b', 'b c']
 """
+# Note: uses import ... as for sake of reload
 # TPO: TODO: 
 # - add logging
 # - replace all_ngrams with min_ngram_size
@@ -24,50 +26,77 @@ from collections import namedtuple
 import re
 import sys  # python2 support
 from threading import RLock
+import html
 
 # Installed modules
 ## TODO: find better documented memoization package
 from cachetools import LRUCache, cached  # python2 support
+import nltk
 from nltk.stem import SnowballStemmer
-from six.moves.html_parser import HTMLParser  # python2 support
-from sklearn.feature_extraction.text import CountVectorizer
+## OLD: from sklearn.feature_extraction.text import CountVectorizer
+CountVectorizer = None
 from stop_words import get_stop_words
 
 # Local modules
 from mezcla import debug
+from mezcla.my_regex import my_re, REGEX_TRACE_LEVEL
 from mezcla import system
+from mezcla.tfidf.config import BASE_DEBUG_LEVEL as BDL
 from mezcla.tfidf.dockeyword import DocKeyword
 
-unescape = HTMLParser().unescape
+unescape = html.unescape
+
 
 # TPO: Allow word splitting pattern to be overwritten.
 # TODO: Get WORD_REGEX from environment.
 # TODO: Have option to ignore words with specific punctuation (or to include).
 # TODO: strip period to allow for ngrams with abbreviations (e.g., "Dr. Jones").
-SPLIT_WORDS = system.getenv_bool("SPLIT_WORDS", False,
-                                 "Split by word token instead of whitespace")
-BAD_WORD_PUNCT_REGEX = system.getenv_text("BAD_WORD_PUNCT_REGEX", r"[^a-zA-Z0-9_'$ -]",
-                                          "Regex for punctuation not in words")
-SKIP_WORD_CLEANING = system.getenv_bool("SKIP_WORD_CLEANING", False,
-                                        "Omit word punctuation cleanup")
-## OLD:
-## SKLEARN_TOKENIZER = system.getenv_bool("SKLEARN_TOKENIZER", False,
-##                                        "Use sklearn CountVectorizer for ngrams")
-USE_SKLEARN_COUNTER = system.getenv_bool("USE_SKLEARN_COUNTER", False,
-                                         "Use sklearn CountVectorizer for ngrams")
-
+SPLIT_WORDS = system.getenv_bool(
+    "SPLIT_WORDS", False,
+    description="Split by word token instead of whitespace")
+TFIDF_ALLOW_PUNCT = system.getenv_bool(
+    "TFIDF_ALLOW_PUNCT", False,
+    description="Allow ngrams to include words with punctuation")
+TFIDF_SENT_SPLITTER = system.getenv_bool(
+    "TFIDF_SENT_SPLITTER", False,
+    description="Use NLTK for splitting sentences for TF/IDF")
+USE_SIMPLE_SENT_SPLITTER = (not TFIDF_SENT_SPLITTER)
+BAD_WORD_PUNCT_REGEX = system.getenv_text(
+    "BAD_WORD_PUNCT_REGEX", r"[^a-zA-Z0-9_'$ -]",
+    description="Regex for punctuation not in words")
+SKIP_WORD_CLEANING = system.getenv_bool(
+    "SKIP_WORD_CLEANING", False,
+    description="Omit word punctuation cleanup")
+USE_SKLEARN_COUNTER = system.getenv_bool(
+    "USE_SKLEARN_COUNTER", False,
+    description="Use sklearn CountVectorizer for ngrams")
+TFIDF_PRESERVE_CASE = system.getenv_bool(
+    "TFIDF_PRESERVE_CASE", False,
+    description="Preserve case in TFIDF ngrams")
+TFIDF_LANGUAGE = system.getenv_value(
+    "TFIDF_LANGUAGE", None,
+    description="Language for text preprocessing")
 
 if SPLIT_WORDS:
-    debug.trace(1, "Splitting by word token (not whitespace)\n")
-    ## BAD: True
+    debug.trace(2, "FYI: Splitting by word token (not whitespace)\n")
+    debug.assertion(not TFIDF_ALLOW_PUNCT)
 WORD_REGEX = r'\w+' if SPLIT_WORDS else r'\S+'
 
+
 def handle_unicode(text):
     """Needed for the description fields."""
+    # EX: handle_unicode("Predactica\u2122") => "Predactica \u2122"
     if re.search(r'\\+((u([0-9]|[a-z]|[A-Z]){4}))', text):
         text = text.encode('utf-8', 'ignore').decode('unicode-escape', 'ignore')
     text = re.sub(r'\\n', '\n', text)
     text = re.sub(r'\\t', '\t', text)
+    # Put space around non-ascii unicode characters
+    # note: excludes special punctuation like U+00A9 (©)
+    EXT_ASC_CH = "\u0000-\u007F\u00C0-\u0100"
+    if re.search(fr'[^{EXT_ASC_CH}]', text):
+        text = re.sub(fr'([{EXT_ASC_CH}]) ([^{EXT_ASC_CH}]) ([{EXT_ASC_CH}])',
+                      r'\1 \2 \3', f" {text} ", flags=re.VERBOSE)
+        text = text.strip(" ")
     return text
 
 
@@ -80,25 +109,26 @@ def handle_html_unquote(text):
 
 def handle_mac_quotes(text):
     """Handle the unfortunate non-ascii quotes OSX inserts."""
-    text = text.replace(u'“', u'"').replace(u'”', u'"')\
-        .replace(u'‘', u"'").replace(u'’', u"'")
+    text = text.replace('“', '"').replace('”', '"')\
+        .replace('‘', "'").replace('’', "'")
     return text
 
 
 def handle_text_break_dash(text):
     """Convert text break dashes into semicolons to simplify things.
 
-    Example:
+    For example,
         "She loved icecream- mint chip especially"
         "She loved icecream - mint chip especially"
-        both convert to
-        "She loved icecream; mint chip especially"
+    will both convert to
+        "She loved icecream; mint chip especially".
 
-        However,
+    However,
         "The 27-year-old could eat icecream any day"
-        will not be changed.
+    will not be changed.
     """
-    return re.sub(r'\s+-\s*|\s*-\s+', ';', text)
+    # EX: handle_text_break_dash("a - b") => "a; b"
+    return re.sub(r'\s+-\s*|\s*-\s+', '; ', text)
 
 
 def clean_text(raw_text):
@@ -116,13 +146,14 @@ def clean_text(raw_text):
         text = handle_html_unquote(text)
         text = handle_mac_quotes(text)
         text = handle_text_break_dash(text)
-        text = text.lower()
+        if not TFIDF_PRESERVE_CASE:
+            text = text.lower()
 
-        ## BAD: regex_subs = ['\t', '\n', '\r', '\s+', '&']
+        ## TODO3: make &-stripping optional
         regex_subs = ['\t', '\n', '\r', r'\s+', '&']
         for regex_sub in regex_subs:
             text = re.sub(regex_sub, ' ', text)
-    debug.trace(7, "clean_text({raw_text}) => {text}")
+    debug.trace(BDL + 2, f"clean_text({raw_text!r}) => {text!r}")
     return text
 
 
@@ -134,7 +165,7 @@ def create_stemmer(language):
     def stem_wordform(wordform):
         """Returned root of WORDFORM"""
         root = stemmer.stem(wordform)
-        debug.trace_fmt(8, "stem_wordform({wf}) => {r}", wf=wordform, r=root)
+        debug.trace_fmt(BDL + 3, "stem_wordform({wf}) => {r}", wf=wordform, r=root)
         return root
 
     # Do sanity check
@@ -156,14 +187,20 @@ class Preprocessor(object):
     If a stopwords file is provided, it will remove stopwords.
 
     Example:
-        >>> processor = Preprocessor('english_stopwords.txt')
-        >>> processor.clean('He was an interesting fellow.')
-        "was interesting fellow."
+        >>> import mezcla.tfidf.preprocess as mtp
+        >>> processor = mtp.Preprocessor(language='english')
+        >>> processor.handle_stopwords('She is an interesting gal.', clean=True)
+        'interesting gal.'
+        >>> sp_processor = mtp.Preprocessor(language='spanish')
+        >>> sp_processor.handle_stopwords('She is an interesting gal.', clean=True)
+        'she is an interesting gal.'
     """
 
     stopwords = set()
     contractions = r"(n't|'s|'re|'ll)$"
-    negative_gram_breaks = r'[^:;!^,\?\.\[|\]\(|\)"`]+'
+    ## OLD: negative_gram_breaks = r'[^:;!^,\?\.\[|\]\(|\)"`]+'
+    ## TODO3: document the first regex (e.g., apparent redundancy w/ '|')
+    negative_gram_breaks = r'[^:;!^,\?\.\[|\]\(|\)"`]+' if not TFIDF_ALLOW_PUNCT else  r'[^:;!\?\.]+'
     supported_languages = (
         'danish', 'dutch', 'english', 'finnish', 'french', 'german', 'hungarian',
         'italian', 'kazakh', 'norwegian', 'porter', 'portuguese', 'romanian',
@@ -172,7 +209,7 @@ class Preprocessor(object):
 
     def __init__(self, min_ngram_size=None, max_ngram_size=None,
                  language=None, stopwords_file=None, stemmer=None,
-                 gramsize=None, all_ngrams=None):
+                 gramsize=None, all_ngrams=None, use_sklearn_counter=None):
         """Preprocessor must be initalized for use if using stopwords.
 
         stopwords_file (filename): contains stopwords, one per line
@@ -190,6 +227,8 @@ class Preprocessor(object):
         stopwords_file (filename):
             Provide a list of stopwords. If used in addition to "language", the
             provided stopwords file overrides the default.
+        language:
+            name of language from NLTK (n.b., use empty string or 'n/a' for none)
         stemmer (function):
             A function that takes in a single argument (str) and returns a string
             as the stemmed word. Overrides the default behavior if specified.
@@ -204,8 +243,12 @@ class Preprocessor(object):
             will be run.
             Note: deprecated (use min_ngram_size instead).
         """
+        # Note: stemmer is no-op if no language specified
         self.__stemmer = None
-        if language:
+        # Note: language can be empty string to block defaults
+        if language is None:
+            language = TFIDF_LANGUAGE
+        if (language and (language != "n/a")):
             debug.assertion(language in self.supported_languages)
             if language in SnowballStemmer.languages:
                 self.__stemmer = create_stemmer(language)
@@ -215,14 +258,16 @@ class Preprocessor(object):
         if stemmer:
             self.__stemmer = stemmer
         if not self.__stemmer:
-            debug.trace(3, "Warning: defining no-op stemmer in Preprocessor")
+            debug.trace(BDL - 2, "Warning: defining no-op stemmer in Preprocessor")
             self.__stemmer = lambda x: x  # no change to word
+        if use_sklearn_counter is None:
+            use_sklearn_counter = USE_SKLEARN_COUNTER
+        self.use_sklearn_counter = USE_SKLEARN_COUNTER
         debug.assertion(not (gramsize and max_ngram_size))
         debug.assertion(not (all_ngrams and min_ngram_size))
-        ## OLD: self.__gramsize = gramsize
-        self.__gramsize = (max_ngram_size or gramsize)
+        self.__gramsize = (max_ngram_size or gramsize or 1)
         self.__all_ngrams = all_ngrams
-        self.__min_ngram_size = (min_ngram_size or gramsize)
+        self.__min_ngram_size = (min_ngram_size or gramsize or 1)
 
     @property
     def gramsize(self):
@@ -245,15 +290,18 @@ class Preprocessor(object):
         return self.__min_ngram_size
 
     def _load_stopwords(self, filename):
-        with open(filename) as f:
+        with system.open_file(filename) as f:
             words = []
             for line in f:
                 words.append(line.strip())
         self.stopwords = set(words)
 
-    def handle_stopwords(self, text):
+    def handle_stopwords(self, text, clean=None):
         """Remove stop words from the text."""
         out = []
+        if clean:
+            text = clean_text(text)
+        ## TODO2: lower().split()
         for word in text.split(' '):
             # Remove common contractions for stopwords when checking list
             check_me = re.sub(self.contractions, '', word)
@@ -304,12 +352,18 @@ class Preprocessor(object):
             ['all', 'the', 'car', 'were', 'honk', 'their', 'horn']
         """
         ## TODO: yield_method = self.slow_yield_keywords if not USE_SKLEARN_COUNTER else self.yield_sklearn_keywords
-        if USE_SKLEARN_COUNTER:
+        if self.use_sklearn_counter:
             result = self.quick_yield_keywords(raw_text, document=document)
         else:
             result = self.full_yield_keywords(raw_text, document=document)
         return result
 
+    def re_search(self, regex, text):
+        """Invoke my_re.search with higher tracing"""
+        debug.reference_var(self)
+        TRACE_LEVEL = (REGEX_TRACE_LEVEL + 1)
+        return my_re.search(regex, text, base_trace_level=TRACE_LEVEL)
+    
     def full_yield_keywords(self, raw_text, document=None):
         """Full-featured version of keyword generation, including support for offsets"""
         if sys.version_info[0] < 3:  # python2 support
@@ -326,10 +380,12 @@ class Preprocessor(object):
             gramlist = range(self.min_ngram_size, self.gramsize + 1)
         if not gramlist:
             gramlist = [self.gramsize]
-        ## DEBUG: sys.stderr.write("gramlist={gl}\n".format(gl=gramlist))
-        debug.trace_fmt(7, "gramlist={gl}\n", gl=gramlist)
+        debug.trace_fmt(BDL + 2, "gramlist={gl}", gl=gramlist)
 
-        for sentence in positional_splitter(self.negative_gram_breaks, raw_text):
+        sentence_split = (positional_splitter(self.negative_gram_breaks, raw_text)
+                          if USE_SIMPLE_SENT_SPLITTER else nltk_sent_splitter(raw_text))
+        for sentence in sentence_split:
+            debug.trace_expr(BDL + 2, sentence.text)
             words = positional_splitter(WORD_REGEX, sentence.text)
             # Remove all stopwords
             words_no_stopwords = []
@@ -338,6 +394,9 @@ class Preprocessor(object):
                 check_me = re.sub(self.contractions, '', w.text)
                 if check_me not in self.stopwords:
                     words_no_stopwords.append(w)
+            if debug.debugging(BDL + 2):
+                words_text_no_stopwords = [w.text for w in words_no_stopwords]
+                debug.trace_expr(BDL + 2, words_text_no_stopwords)
 
             # Make the ngrams
             # TODO: make sure stripped stopwords block ngram (e.g. "dog and cat" =/=> "dog cat")
@@ -351,20 +410,36 @@ class Preprocessor(object):
                                       if len(data[pos:pos + gramsize]) == gramsize]
                     for word_list in text_in_chunks:
                         word_text = ' '.join([self.stem_term(w.text) for w in word_list])
-                        if (SPLIT_WORDS or (not re.search(BAD_WORD_PUNCT_REGEX, word_text))):
+                        ## OLD: if (SPLIT_WORDS or (not re.search(BAD_WORD_PUNCT_REGEX, word_text))):
+                        exclude = self.re_search(BAD_WORD_PUNCT_REGEX, word_text)
+                        if TFIDF_ALLOW_PUNCT and exclude:
+                            # Include unless punctuation starts or ends the text or is after a space
+                            # TODO2: add BAD_NGRAM_PUNCT_REGEX (e.g., "[;!?]")
+                            exclude = (self.re_search("^" + BAD_WORD_PUNCT_REGEX, word_text) or
+                                       self.re_search(BAD_WORD_PUNCT_REGEX + "$", word_text) or
+                                       self.re_search(" " + BAD_WORD_PUNCT_REGEX, word_text))
+                        if not exclude:
                             word_global_start = sentence.start + word_list[0].start
                             word_global_end = sentence.start + word_list[-1].end
                             yield DocKeyword(word_text, document=document, start=word_global_start, end=word_global_end)
+                        else:
+                            debug.trace(BDL + 3, f"Ignoring {gramsize}-gram {word_text!r}")
         return
 
     def quick_yield_keywords(self, raw_text, document=None):
-        """Quick version for yielfing keywords, using sklearn for ngram generation
+        """Quick version for yielding keywords, using sklearn for ngram generation
         Note: the DocKeyword objects don't include offset information"""
         # TODO1: support BAD_WORD_PUNCT_REGEX
-
+        # Do dynamic load(s)
+        global CountVectorizer
+        if not CountVectorizer:
+            # pylint: disable=import-outside-toplevel,redefined-outer-name
+            from sklearn.feature_extraction.text import CountVectorizer
         vectorizer = CountVectorizer(ngram_range=(self.min_ngram_size, self.gramsize))
         analyzer = vectorizer.build_analyzer()
+        ## DEBUG: debug.trace_expr(8, analyzer)
         for ngram in analyzer(raw_text):
+            ## DEBUG: debug.trace_expr(9, ngram)
             yield DocKeyword(ngram, document=document)
         return
 
@@ -379,6 +454,7 @@ def positional_splitter(regex, text):
         To split on whitespace, you match:
         r'\S+'  <-- "a chain of anything that's NOT whitespace"
     """
+    ## TODO2: treat -- as clausal punctuation if TFIDF_ALLOW_PUNCT
     for res in re.finditer(regex, text):
         ## TODO: drop try/except
         try:
@@ -388,12 +464,23 @@ def positional_splitter(regex, text):
     return
 
 
+def nltk_sent_splitter(text):
+    """Yield sentence chunks as well as their location."""
+    # Note: Similar to positional_splitter but uses NLTK instead of negative_gram_breaks regex
+    offset = 0
+    for sent in nltk.tokenize.sent_tokenize(text):
+        end = offset + (1 + len(sent))
+        yield PositionalWord(sent, offset, end)
+        offset = end
+    return
+
+
 def main():
     """Entry point for script: just runs a simple test"""
     text = "my man fran is not a man"
     p = Preprocessor(gramsize=2)
-    debug.trace_object(4, p)
-    ngrams = list([p.text for p in p.yield_keywords(text)])
+    debug.trace_object(BDL - 2, p)
+    ngrams = list(p.text for p in p.yield_keywords(text))
     debug.trace_expr(3, text, ngrams)
     debug.assertion("my man" in ngrams)
     debug.assertion("my man fran" not in ngrams)
@@ -402,5 +489,5 @@ def main():
 #-------------------------------------------------------------------------------
     
 if __name__ == '__main__':
-    system.print_stderr(f"Warning: {__file__} is not intended to be run standalone. A simple test willl be run.")
+    system.print_stderr(f"Warning: {__file__} is not intended to be run standalone. A simple test will be run.")
     main()
