@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 # tpo_common.py: Python module with common utilities mostly for debugging,
 # mostly superceded by debug.py, system.py, and misc_util.py """
 #
@@ -23,6 +23,11 @@
 #   arguments to functions that do nothing in release code.
 # - Includes support for overriding stderr (e.g., for use with embedded apps as in adhoc/optimize_company_extraction.py's invocation of extract_company_info.py).
 #
+# TODO1:
+# - *** Make sure any function not mapping directly to debug or system traces out
+#   with "tpo" for grep purposes (e.g.,  tpo.debug_format in debug_format). ***
+#   This will help flag functions to be ported (e.g., redirect_stderr).
+#
 # TODO:
 # - Change all functions to using docstrings for comments already!!!
 # - Add support for using pprint module
@@ -70,6 +75,7 @@ except ImportError:
 from mezcla import debug
 from mezcla.debug import trace_level as debug_level
 from mezcla import system
+from mezcla import text_utils
             
 # Defaults for globals
 ## OLD:
@@ -221,7 +227,7 @@ USE_SIMPLE_FORMAT = (sys.version_info[0] <= 2) and (sys.version_info[1] <= 5)
 ## 
 ##     def debug_timestamp():
 ##         """Return timestamp for use in debugging traces"""
-##         # EX: debug_timestamp() => "2015-01-18 16:39:45.224768"
+##         # old-EX: debug_timestamp() => "2015-01-18 16:39:45.224768"
 ##         # TODO: use format compatible with logging (e.g., comma in place of period before micrososeconds)
 ##         return to_string(datetime.now())
 ## 
@@ -479,6 +485,8 @@ def debug_format(text, level=1, skip_newline=False, **namespace):
     """Version of debug_print that expands TEXT using format.
     Note: Exceptions are ignored (to encourage tracing, not discourage)."
     """
+    debug.trace(6, f"tpo.debug_format({text}, {skip_newline}, ...)",
+                skip_sanity_checks=True)
     ## TODO: return debug.trace(no_eol=skip_newline, level, text, **namespace)
     # NOTE: String values from namespace need to be in UTF-8 format.
     # TODO: rename as debug_print_format as not just formatting the text
@@ -493,9 +501,11 @@ def debug_format(text, level=1, skip_newline=False, **namespace):
 
 def debug_timestamp():
     """Return timestamp for use in debugging traces"""
-    # EX: debug_timestamp() => "2015-01-18 16:39:45.224768"
+    # Example: debug_timestamp() => "2015-01-18 16:39:45.224768"
     # TODO: use format compatible with logging (e.g., comma in place of period before micrososeconds)
     return debug.timestamp()
+#
+# EX: re.sub(r"[0-9]", "N", debug_timestamp()) => "NNNN-NN-NN NN:NN:NN.NNNNNN"
 
 
 def debug_raise(level=1):
@@ -545,13 +555,13 @@ def trace_value(value, level=5, label=None):
 
 
 def trace_current_context(level=QUITE_DETAILED, label=None, 
-                          show_methods_etc=False):
+                          show_methods_etc=False, **kwargs):
     """Traces out current context (local and global variables), with output
     prefixed by "LABEL context" (e.g., "current context: {\nglobals: ...}").
     Notes: By default the debugging level must be quite-detailed (6).
     If the debugging level is higher, the entire stack frame is traced.
     Also, methods are omitted by default."""
-    return debug.trace_current_context(level=level, label=label, show_methods_etc=show_methods_etc)
+    return debug.trace_current_context(level=level, label=label, show_methods_etc=show_methods_etc, indirect=True, **kwargs)
 
 
 def during_debugging(expression=True):
@@ -604,7 +614,7 @@ def normalize_unicode(text, encoding="utf8"):
 #
 def _normalize_unicode(text, encoding="utf8"):
     """Implementation of normalize_unicode (q.v.)"""
-    # EX: normalize_unicode(u'\u1234') => '\xe1\x88\xb4'
+    # old-EX: _normalize_unicode(u'\u1234') => '\xe1\x88\xb4'
     # Note: As this is used in suport for debug_print, no tracing is done.
     # TODO: work out more-untuitive name
     result = text
@@ -616,7 +626,7 @@ def _normalize_unicode(text, encoding="utf8"):
 
 def ensure_unicode(text, encoding="utf8"):
     """Ensures TEXT is encoded as unicode, using ENCODING (e.g., UTF8). Note: this is a no-op in Python 3 or higher"""
-    # EX: ensure_unicode('\xe1\x88\xb4') => u'\u1234'
+    # old-EX: ensure_unicode('\xe1\x88\xb4') => u'\u1234'
     result = _ensure_unicode(text, encoding)
     debug_format("ensure_unicode({t}) => {r}", 10, 
                  t=text, r=result)
@@ -649,7 +659,7 @@ def redirect_stderr(filename):
     """Redirects error output to FILENAME"""
     global stderr
     assert(stderr == sys.stderr)
-    stderr = open(filename, "w")
+    stderr = system.open_file(filename, "w")
     assert(stderr)
 
 
@@ -657,7 +667,8 @@ def restore_stderr():
     """Restores error output to system stderr, closing handle for redirection"""
     global stderr
     assert(stderr != sys.stderr)
-    stderr.close()
+    if stderr:
+        stderr.close()
     stderr = sys.stderr
 
 
@@ -714,8 +725,10 @@ def register_env_option(var, description, default):
 
 def get_registered_env_options():
     """Returns list of environment options registered via register_env_option"""
-    option_names = [k for k in env_options if (env_options[k] and env_options[k].strip())]
-    debug_format("get_registered_env_options() => {option_names}", 5)
+    ## OLD: option_names = [k for k in env_options if (env_options[k] and env_options[k].strip())]
+    option_names = [k for (k, v) in env_options.items() if (v and v.strip())]
+    ## OLD: debug_format("get_registered_env_options() => {option_names}", 5)
+    debug_print(f"get_registered_env_options() => {option_names}", 5)
     return option_names
 
 
@@ -740,7 +753,8 @@ def get_environment_option_descriptions(include_all=None, include_default=None,
         default_spec = default_spec.replace("\n", "\\n")
         return (opt, desc_spec + indent + default_spec)
     #
-    option_descriptions = [_format_env_option(opt) for opt in env_options if (env_options[opt] or include_all)]
+    ## OLD: option_descriptions = [_format_env_option(opt) for opt in env_options if (env_options[opt] or include_all)]
+    option_descriptions = [_format_env_option(opt) for (opt, v) in env_options.items() if (v or include_all)]
     debug_format("get_environment_option_descriptions() => {od}", 5,
                  od=option_descriptions)
     return option_descriptions
@@ -992,7 +1006,7 @@ def load_object(filename):
     """Load object data from FILENAME in pickle format"""
     debug_print("Loading object from %s" % filename, 3)
     object_data = None
-    f = open(filename, 'rb')
+    f = system.open_file(filename, 'rb')
     if f:
         object_data = pickle.load(f)
         f.close()
@@ -1002,7 +1016,7 @@ def load_object(filename):
 def store_object(filename, object_data):
     """Store OBJECT_DATA in FILENAME using pickle format"""
     debug_print("Saving object to %s" % filename, 3)
-    f = open(filename, 'wb')
+    f = system.open_file(filename, 'wb')
     if f:
         pickle.dump(object_data, f)
         f.close()
@@ -1030,7 +1044,7 @@ def create_lookup_table(filename, use_linenum=False):
     lookup_hash = {} if (not use_linenum) else OrderedDict()
     f = None
     try:
-        f = open(filename)
+        f = system.open_file(filename)
         line_num = 0
         for line in f:
             line = line.strip("\n")
@@ -1065,7 +1079,7 @@ def create_boolean_lookup_table(filename):
     # TODO: allow for tab-delimited value to be ignored
     debug_print("create_boolean_lookup_table(%s)" % filename, 4)
     lookup_hash = {}
-    f = open(filename)
+    f = system.open_file(filename)
     for line in f:
         key = line.strip().lower()
         lookup_hash[key] = True
@@ -1239,7 +1253,7 @@ def round_num(num, precision=None, zero_fill=True):
 
 def round_nums(numbers, precision=None, zero_fill=True):
     """Rounds each of the NUMBERS to PRECISION places (normally 3): returned as list of strings, optionally zero-filled on the right"""
-    # EX: round_nums([0.333333, 0.666666, 0.99999]) = ['0.333', '0.667', '1.000']
+    # EX: round_nums([0.333333, 0.666666, 0.99999]) => ['0.333', '0.667', '1.000']
     return [round_num(v, precision, zero_fill) for v in numbers]
 
 
@@ -1274,51 +1288,52 @@ def normalize(num_list):
 
 
 def is_numeric(text):
-    """Indicates whether TEXT represents a number (integer or float)"""
-    ## TODO: rename as is_number to avoid confusion with English numeric a la numeral (e.g., "two')
-    # EX: is_numeric("123") => True
-    # EX: is_numeric("one") => False
-    numeric = False
-    try:
-        _value = float(text)
-        numeric = True
-    except ValueError:
-        _value = None
-    debug_format("is_numeric({t}) => {n}; value={v}", 8, t=text, n=numeric, v=_value)
-    return numeric
+    """Indicates whether TEXT represents a number (integer or float)
+    Deprecated: uses version in text_utils.py"
+    """
+    debug.trace(3, "Warning: in deprecated text_utils.is_numeric")
+    return text_utils.is_numeric(text)
 
 
-def safe_int(numeric, default_value=0, base=10):
-    """Returns NUMERIC interpreted as a int, using DEFAULT_VALUE (0) if not
-    numeric and interpretting value as optional BASE (10)"""
-    try:
-        result = int(numeric, base)
-    except (TypeError, ValueError):
-        debug_format("Warning: Exception converting integer numeric ({num}): {exc}", 4,
-                     num=numeric, exc=str(sys.exc_info()))
-        result = default_value
-    debug_format("safe_int({n}, [{df}, {b}]) => {r})", 7,
-                 n=numeric, df=default_value, b=base, r=result)
-    return result
+## OLD:
+## 
+## def safe_int(numeric, default_value=0, base=10):
+##     """Returns NUMERIC interpreted as a int, using DEFAULT_VALUE (0) if not
+##     numeric and interpretting value as optional BASE (10)"""
+##     try:
+##         result = int(numeric, base)
+##     except (TypeError, ValueError):
+##         debug_format("Warning: Exception converting integer numeric ({num}): {exc}", 4,
+##                      num=numeric, exc=str(sys.exc_info()))
+##         result = default_value
+##     debug_format("safe_int({n}, [{df}, {b}]) => {r})", 7,
+##                  n=numeric, df=default_value, b=base, r=result)
+##     return result
+## 
+## 
+## def safe_float(numeric, default_value=0.0):
+##     """Returns NUMERIC interpreted as a float, using DEFAULT_VALUE (0.0) if not numeric"""
+##     try:
+##         result = float(numeric)
+##     except ValueError:
+##         debug_format("Warning: Exception converting float numeric ({num}): {exc}", 4,
+##                      num=numeric, exc=str(sys.exc_info()))
+##         result = default_value
+##     debug_format("safe_int({n}, {df}) => {r})", 7,
+##                  n=numeric, df=default_value, r=result)
+##     return result
+##
+## def reference_variables(*args):
+##     """Dummy function used for referencing variables"""
+##     debug_print("reference_variables%s" % str(args), 9)
+##     return
+##
 
+safe_int = system.to_int
 
-def safe_float(numeric, default_value=0.0):
-    """Returns NUMERIC interpreted as a float, using DEFAULT_VALUE (0.0) if not numeric"""
-    try:
-        result = float(numeric)
-    except ValueError:
-        debug_format("Warning: Exception converting float numeric ({num}): {exc}", 4,
-                     num=numeric, exc=str(sys.exc_info()))
-        result = default_value
-    debug_format("safe_int({n}, {df}) => {r})", 7,
-                 n=numeric, df=default_value, r=result)
-    return result
+safe_float = system.to_float
 
-
-def reference_variables(*args):
-    """Dummy function used for referencing variables"""
-    debug_print("reference_variables%s" % str(args), 9)
-    return
+reference_variables = debug.reference_var
 
 #------------------------------------------------------------------------
 # Memomization support (i.e., functiona result caching), based on 
@@ -1330,21 +1345,27 @@ def reference_variables(*args):
 #
 #    @memodict
 #    def fubar(word):
+#        """Fouled up"""
 #        result = ...
 #        return result
 #
 
 def memodict(f):
-    """Memoization decorator for a function taking a single argument"""
-    class _memodict(dict):
-        """Internal class for implementing memoization"""
-        #
-        def __missing__(self, key):
-            """Invokes function to produce value if arg not in hash"""
-            ret = self[key] = f(key)
-            return ret
-    #
-    return _memodict().__getitem__
+    """Memoization decorator for a function taking a single argument
+    Warning: deprecated; uses version in system.py instead
+    """
+    return system.memodict(f)
+    ## OLD
+    ## """Memoization decorator for a function taking a single argument"""
+    ## class _memodict(dict):
+    ##     """Internal class for implementing memoization"""
+    ##     #
+    ##     def __missing__(self, key):
+    ##         """Invokes function to produce value if arg not in hash"""
+    ##         ret = self[key] = f(key)
+    ##         return ret
+    ## #
+    ## return _memodict().__getitem__
 
 #------------------------------------------------------------------------
 
